package org.springframework.boot.loader.archive;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.FileSystem;
import java.nio.file.Files;
import java.nio.file.OpenOption;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.nio.file.attribute.FileAttribute;
import java.nio.file.attribute.PosixFilePermission;
import java.nio.file.attribute.PosixFilePermissions;
import java.util.EnumSet;
import java.util.Iterator;
import java.util.UUID;
import java.util.jar.JarEntry;
import java.util.jar.Manifest;
import org.springframework.boot.loader.archive.Archive;
import org.springframework.boot.loader.jar.JarFile;

public class JarFileArchive implements Archive {
  private static final String UNPACK_MARKER = "UNPACK:";
  
  private static final int BUFFER_SIZE = 32768;
  
  private static final FileAttribute<?>[] NO_FILE_ATTRIBUTES = (FileAttribute<?>[])new FileAttribute[0];
  
  private static final EnumSet<PosixFilePermission> DIRECTORY_PERMISSIONS = EnumSet.of(PosixFilePermission.OWNER_READ, PosixFilePermission.OWNER_WRITE, PosixFilePermission.OWNER_EXECUTE);
  
  private static final EnumSet<PosixFilePermission> FILE_PERMISSIONS = EnumSet.of(PosixFilePermission.OWNER_READ, PosixFilePermission.OWNER_WRITE);
  
  private final JarFile jarFile;
  
  private URL url;
  
  private Path tempUnpackDirectory;
  
  public JarFileArchive(File file) throws IOException {
    this(file, file.toURI().toURL());
  }
  
  public JarFileArchive(File file, URL url) throws IOException {
    this(new JarFile(file));
    this.url = url;
  }
  
  public JarFileArchive(JarFile jarFile) {
    this.jarFile = jarFile;
  }
  
  public URL getUrl() throws MalformedURLException {
    if (this.url != null)
      return this.url; 
    return this.jarFile.getUrl();
  }
  
  public Manifest getManifest() throws IOException {
    return this.jarFile.getManifest();
  }
  
  public Iterator<Archive> getNestedArchives(Archive.EntryFilter searchFilter, Archive.EntryFilter includeFilter) throws IOException {
    return (Iterator<Archive>)new NestedArchiveIterator(this, this.jarFile.iterator(), searchFilter, includeFilter);
  }
  
  @Deprecated
  public Iterator<Archive.Entry> iterator() {
    return (Iterator<Archive.Entry>)new EntryIterator(this.jarFile.iterator(), null, null);
  }
  
  public void close() throws IOException {
    this.jarFile.close();
  }
  
  protected Archive getNestedArchive(Archive.Entry entry) throws IOException {
    JarEntry jarEntry = ((JarFileEntry)entry).getJarEntry();
    if (jarEntry.getComment().startsWith("UNPACK:"))
      return getUnpackedNestedArchive(jarEntry); 
    try {
      JarFile jarFile = this.jarFile.getNestedJarFile(jarEntry);
      return new org.springframework.boot.loader.archive.JarFileArchive(jarFile);
    } catch (Exception ex) {
      throw new IllegalStateException("Failed to get nested archive for entry " + entry.getName(), ex);
    } 
  }
  
  private Archive getUnpackedNestedArchive(JarEntry jarEntry) throws IOException {
    String name = jarEntry.getName();
    if (name.lastIndexOf('/') != -1)
      name = name.substring(name.lastIndexOf('/') + 1); 
    Path path = getTempUnpackDirectory().resolve(name);
    if (!Files.exists(path, new java.nio.file.LinkOption[0]) || Files.size(path) != jarEntry.getSize())
      unpack(jarEntry, path); 
    return new org.springframework.boot.loader.archive.JarFileArchive(path.toFile(), path.toUri().toURL());
  }
  
  private Path getTempUnpackDirectory() {
    if (this.tempUnpackDirectory == null) {
      Path tempDirectory = Paths.get(System.getProperty("java.io.tmpdir"), new String[0]);
      this.tempUnpackDirectory = createUnpackDirectory(tempDirectory);
    } 
    return this.tempUnpackDirectory;
  }
  
  private Path createUnpackDirectory(Path parent) {
    int attempts = 0;
    while (attempts++ < 1000) {
      String fileName = Paths.get(this.jarFile.getName(), new String[0]).getFileName().toString();
      Path unpackDirectory = parent.resolve(fileName + "-spring-boot-libs-" + UUID.randomUUID());
      try {
        createDirectory(unpackDirectory);
        return unpackDirectory;
      } catch (IOException iOException) {}
    } 
    throw new IllegalStateException("Failed to create unpack directory in directory '" + parent + "'");
  }
  
  private void unpack(JarEntry entry, Path path) throws IOException {
    createFile(path);
    path.toFile().deleteOnExit();
    try(InputStream inputStream = this.jarFile.getInputStream(entry); 
        OutputStream outputStream = Files.newOutputStream(path, new OpenOption[] { StandardOpenOption.WRITE, StandardOpenOption.TRUNCATE_EXISTING })) {
      byte[] buffer = new byte[32768];
      int bytesRead;
      while ((bytesRead = inputStream.read(buffer)) != -1)
        outputStream.write(buffer, 0, bytesRead); 
      outputStream.flush();
    } 
  }
  
  private void createDirectory(Path path) throws IOException {
    Files.createDirectory(path, getFileAttributes(path.getFileSystem(), DIRECTORY_PERMISSIONS));
  }
  
  private void createFile(Path path) throws IOException {
    Files.createFile(path, getFileAttributes(path.getFileSystem(), FILE_PERMISSIONS));
  }
  
  private FileAttribute<?>[] getFileAttributes(FileSystem fileSystem, EnumSet<PosixFilePermission> ownerReadWrite) {
    if (!fileSystem.supportedFileAttributeViews().contains("posix"))
      return NO_FILE_ATTRIBUTES; 
    return (FileAttribute<?>[])new FileAttribute[] { PosixFilePermissions.asFileAttribute(ownerReadWrite) };
  }
  
  public String toString() {
    try {
      return getUrl().toString();
    } catch (Exception ex) {
      return "jar archive";
    } 
  }
}
